/*---------------------------------------------------------------------------*
 |        PDFlib - A library for dynamically generating PDF documents        |
 +---------------------------------------------------------------------------+
 |        Copyright (c) 1997-2002 Thomas Merz. All rights reserved.          |
 +---------------------------------------------------------------------------+
 |    This software is NOT in the public domain.  It can be used under two   |
 |    substantially different licensing terms:                               |
 |                                                                           |
 |    The commercial license is available for a fee, and allows you to       |
 |    - ship a commercial product based on PDFlib                            |
 |    - implement commercial Web services with PDFlib                        |
 |    - distribute (free or commercial) software when the source code is     |
 |      not made available                                                   |
 |    Details can be found in the file PDFlib-license.pdf.                   |
 |                                                                           |
 |    The "Aladdin Free Public License" doesn't require any license fee,     |
 |    and allows you to                                                      |
 |    - develop and distribute PDFlib-based software for which the complete  |
 |      source code is made available                                        |
 |    - redistribute PDFlib non-commercially under certain conditions        |
 |    - redistribute PDFlib on digital media for a fee if the complete       |
 |      contents of the media are freely redistributable                     |
 |    Details can be found in the file aladdin-license.pdf.                  |
 |                                                                           |
 |    These conditions extend to ports to other programming languages.       |
 |    PDFlib is distributed with no warranty of any kind. Commercial users,  |
 |    however, will receive warranty and support statements in writing.      |
 *---------------------------------------------------------------------------*/

/* pdflib.i
 *
 * PDFlib interface file for SWIG
 *
 */

#if defined(SWIGRUBY)
%module pdflib_ruby
#else
%module pdflib_XXX
#endif

/* Fetch SWIG's exception handling mechanism */
/*
%include exception.i
*/

%text %{
This is only a brief summary of the PDFlib API interface.
For a more complete description of the functions see the PDFlib API
reference manual.

Most functions do not return any value. For those functions which do
return a value, the meaning of the returned value is explained in the
function description below.

\\
%}

/*
 * The JNI currently doesn't support initialization routines - we handle
 * this ourselves with a static method in the Java pragma further below.
 */
#if !defined(SWIGJAVA)
%init %{
	/* Boot the PDFlib core */
	PDF_boot();
%}
#endif

#ifdef SWIGTCL
/* to get Tcl_Interp *interp handled correctly */
%import typemaps.i

/* THE PDFLIB_VERSIONSTRING macro will be substituted in the wrapper file */
%init %{
	/* Tell Tcl which package we are going to define */
	Tcl_PkgProvide(interp, "pdflib", PDFLIB_VERSIONSTRING);
%}
#endif

/* The version number of the shared library will be patched by make */
%pragma(java) module = "
    // The initialization code for loading the PDFlib shared library.
    // The library name will be transformed into something platform-
    // specific by the VM.
    static {
	try {
	    System.loadLibrary(\"pdf_java\");
	} catch (UnsatisfiedLinkError e) {
	    System.err.println(
	    \"Cannot load the PDFlib shared library for Java.\n\" +
	    \"Make sure that LD_LIBRARY_PATH (Unix) or PATH (Win32)\n\" +
	    \"contains the current directory\n\" + e);
	    System.exit(1);
	}
	PDF_boot();
    }
";

/* The following block will be copied verbatim to the wrapper files. */
%{
#include <setjmp.h>

/* The following sick stuff is an attempt to make recent Perl versions
 * work with recent SWIG versions compiled with recent MS VC++ versions.
 */
#ifdef _MSC_VER
    #undef setjmp
    #define setjmp _setjmp
    #undef longjmp
#endif

#include "pdflib.h"

#ifdef PDF_NO_LONGER_USED
/* Map PDFlib errors to SWIG exceptions */
static const int pdf_swig_exceptions[] = {
    0,
    SWIG_MemoryError,	/* PDF_MemoryError    1 */
    SWIG_IOError,	/* PDF_IOError        2 */
    SWIG_RuntimeError,	/* PDF_RuntimeError   3 */
    SWIG_IndexError,	/* PDF_IndexError     4 */
    SWIG_TypeError,	/* PDF_TypeError      5 */
    SWIG_DivisionByZero,/* PDF_DivisionByZero 6 */
    SWIG_OverflowError,	/* PDF_OverflowError  7 */
    SWIG_SyntaxError,	/* PDF_SyntaxError    8 */
    SWIG_ValueError,	/* PDF_ValueError     9 */
    SWIG_SystemError,	/* PDF_SystemError   10 */
    SWIG_UnknownError,	/* PDF_NonfatalError 11 */ /* PDFlib-special */
    SWIG_UnknownError,	/* PDF_UnknownError  12 */
};
#endif

/* SWIG exception handling is currently not thread-safe! */
static jmp_buf		exception_buffer;
static int		exception_status;
static char		error_message[256];

/* Exception handling for SWIG. */
#define try		if ((exception_status = setjmp(exception_buffer)) == 0)
#define catch(error)	else if (exception_status == error)
#define throw(error)	longjmp(exception_buffer, error)

static void
pdf_swig_errorhandler(PDF *p, int type, const char* shortmsg)
{
    sprintf(error_message, "PDFlib: %s\n", shortmsg);

    /* Issue a warning message and continue for non-fatal errors */
    if (type == PDF_NonfatalError) {
	fprintf(stderr, error_message);
	return;
    }
	
    /* Clean up PDFlib internals and delete p in all other cases */
    if (p != NULL)
	PDF_delete(p);

    /* ...and throw an exception */
    throw(type);
}

%}

#ifdef SWIGJAVA
%{
/* Map PDFlib errors to Java exceptions */
static const char *pdf_java_exceptions[] = {
NULL,
"java/lang/OutOfMemoryError",		/* PDF_MemoryError    1 */
"java/io/IOException",			/* PDF_IOError        2 */
"java/lang/IllegalArgumentException",	/* PDF_RuntimeError   3 */
"java/lang/IndexOutOfBoundsException",	/* PDF_IndexError     4 */
"java/lang/ClassCastException",		/* PDF_TypeError      5 */
"java/lang/ArithmeticException",	/* PDF_DivisionByZero 6 */
"java/lang/ArithmeticException",	/* PDF_OverflowError  7 */
"java/lang/RuntimeException",		/* PDF_SyntaxError    8 */
"java/lang/IllegalArgumentException",	/* PDF_ValueError     9 */
"java/lang/InternalError",		/* PDF_SystemError   10 */
"java/lang/UnknownError",		/* PDF_NonfatalError 11 */
"java/lang/UnknownError",		/* PDF_UnknownError  12 */
};

#define SWIG_exception(type, msg) (*jenv)->ThrowNew(jenv, \
        (*jenv)->FindClass(jenv, pdf_java_exceptions[type]), msg);

%}

#endif	/* SWIGJAVA */

/* This is the error handler which will be used for all languages.
 * The construction is somewhat uglier than the usual try...catch...catch,
 * but this will save us ca. 2000 (!) lines of code in the generated
 * wrapper files since all wrapped functions will be affected by
 * code bloat from the exception handling mechanism.
 */
#ifdef SWIGJAVA

%except {
    try { $function
    } catch;
}
#endif

#ifdef SWIGPERL

%except {
    try { $function
    } {
	SWIG_exception(error_message);
    }
}
#endif

#ifdef SWIGPYTHON

%except {
    try { $function
    } else {
	SWIG_exception();
    }
}
#endif

#ifdef SWIGRUBY

%except {
    try { $function
    } catch;
}
#endif

#ifdef SWIGTCL

%except {
    try { $function
    } catch;
}
#endif

#if !defined(SWIGPYTHON) && !defined(SWIGJAVA) && !defined(SWIGPERL) && !defined(SWIGTCL) && !defined(SWIGRUBY)

%except {
    try { $function }
    else {
	SWIG_exception(pdf_swig_exceptions[exception_status], error_message);
    }
}

#endif

%{
/* export the PDFlib routines to the shared library */
#ifdef __MWERKS__
#pragma export on
#endif

%}

/* This #include will actually be carried out by SWIG. */
%include "pdflib.h"
